/*! \file hchlib_1.0.c
 * httpchanlib 1.0
 * This source code is distributed under GPL GNU Public License version 2 or later.
 */

#include "hchlib.h"
#include "hchlib_encode.h"
#include "hchlib_subf.h"

///////////////////////////////////////////////////////////////////////////////////
//     functions C/S  RETURN VALUES                                              //
//        get_hchlib(), send_hchlib(), postsend_hchlib() and recvsend_hchlib()   //
//                                                                               //
// 	  0 : function returned successfully                                  	 //
//  -33 : GET/POST mismatch (used by server to identify client requests)         //
//	 -1 : ERROR - buffer to send/receive is too long (max is 170kB)          //
//	 -2 : ERROR - mimetype/encoding is too long   			         //
//	 -3 : ERROR - mimetype/encoding unknown                            	 //
//	 -4 : ERROR - write on socket failure                                    //
//	 -5 : ERROR - read from socket failure                        		 //
//	 -6 : WARNING - server has not closed the connection                     //
//	 -7 : WARNING - received buffer is empty                                 //
//	 -8 : ERROR - server response is not a 200 OK                         	 //
//	 -9 : ERROR - client (server) request (response) has an incorrect header //
//  -10 : ERROR - server cannot send a 200 OK                                    //
//  -11 : ERROR - remote side has closed the connection                          //
//  -12 : ERROR - server can't close the connection                              //
//		      							         //
///////////////////////////////////////////////////////////////////////////////////



/*! \brief create_hchlib - crea canale HHTP
 *
 * Funzione per la creazione del canale HTTP verso il server
 * IN:     const char *server, (nome/IP del server),
 *         const char *port    (porta/servizio del server),
 * 				const unsigned int timeout (timeout nella ricezione dell'OK)
 * RETURN: int (socket descriptor)
 */
int 
create_hchlib(const char *server, const char *port, const unsigned int time_val)
{
  int sd;
  struct addrinfo hints, *result, *ressave;
  struct timeval timeout;
  
  timeout.tv_sec = time_val;
  timeout.tv_usec = 0;
  
  if((atoi(port) < 0) || (atoi(port) > 65535))			{hcherror("bad port"); return -1;}
  bzero(&hints, sizeof(hints));
  hints.ai_family = AF_INET;
  hints.ai_socktype = SOCK_STREAM;
  if(getaddrinfo(server, port, &hints, &result) != 0) 		{hcherror("getaddrinfo"); return -2;}

  ressave = result;
  do {
    if((sd = socket(result->ai_family, result->ai_socktype, result->ai_protocol)) < 0) continue;
    if(connect(sd, result->ai_addr, result->ai_addrlen) == 0) break;
    close(sd);
  } while ((result = result->ai_next) != NULL);
  if (result == NULL)																{hcerror("no sockd available"); return -3;}
  freeaddrinfo(ressave);

  setsockopt(sd, SOL_SOCKET, SO_RCVTIMEO, &timeout, sizeof(timeout));

  return sd;
}


/*! \brief post_hchlib() - invia un buffer al server
 *
 * Funzione per inviare il contenuto di un buffer al server.
 *  Il contenuto del buffer puo' essere binario o testo, a seconda del encoding
 *  dato in ingresso verra' encodato 
 *
 *  IN:     int sockd        (socket descriptor, creato con create_hchlib()),
 *   	   const void *buff (buffer da INVIARE),
 *	   int bufflen      (lunghezza del buffer da inviare, NON deve superare i 170kB),
 *		 char *tenc				(transfer encoding)
 *	   char *mime       (mimetype del dato contenuto in buff),
 *  RETURN: int              (0 se OK, numero negativo se errore, vedi tabella)
 */
int 
post_hchlib(int sockd, void *buff, int bufflen, char *tenc, char *mime)
{
  char encoded[MAX_ENC], 
       sendtext[MAX_NET],
       sendtext2[MAX_NET],
       recvtext[MAX_HEAD],
       recvmime[MAX_MIME],
       p[8], mime_p[20], *p4, mimever[4],
       *p1, *p2, *p3, cont[17], 
       status[5], ishttp[9]; 
  int flag=0, k, ip4,
      h, i, j=0,
      enc_len, msg_len,
      ip1, ip2, ip3, code;

  struct textstr{
    char *campo;
  } text[] = { 
    "POST / HTTP/1.0",
    "\r\nContent-type: ",
    "",                     	//2  
    "\r\nContent-lenght: ",
    "",                   	  //4
    "\r\nMIME-Version: ",
    MIMEver,               	  //6
    "\r\nContent-transfer-encoding: ",
    "",                       //8
    "\r\n\r\n",
    "",                      	//10
  }; 
	
  if(bufflen > MAX_DATA)					{hcherror("ERROR post_hchlib() - buffer to send is too long (max is 170kB) : "); return -1;}
  if(strlen(mime) > MAX_MIME)			{hcherror("ERROR post_hchlib() - buffer mimetype is too long (max is 64chars) : "); return -2;}
	if(strlen(tenc) > MAX_TENC)			{hcherror("ERROR post_hchlib() - bad transfer encoding : "); return -2;}

  strslash(mime_p, mime); 
  for(i=0; i<=5; i++){  
    if(strcasecmp(mtype[i].tp, mime_p) == 0){
      flag=1; 
      break;
    }
  }   
  if(flag == 0) 									{hcherror("ERROR post_hchlib() - mimetype unknown : "); return -3; }
	text[2].campo = mime;	
  flag=0;
	for(j=0; j<=4; j++){  
    if(strcasecmp(tenc_struct[j].tp, tenc) == 0){
      flag=1; 
      break;
    }
  } 
	if(flag == 0) 									{hcherror("ERROR post_hchlib() - transfer encoding unknown : "); return -3; }
	text[8].campo = tenc;	

	if((mtype[i].num == 1) && ((tenc_struct[j].num == 1)||(tenc_struct[j].num == 2)))	hcherror("WARNING post_hchlib() - data may be lost with this encoding : ");
	flag = 0;
	if(tenc_struct[j].num == 0){
		enc_len = bufflen;
		text[10].campo = buff;
		flag = 1;
	}	else {
		if(tenc_struct[j].num == 1){
			enc_len=break_line(encoded, buff, bufflen);
		} else {
			if(tenc_struct[j].num == 2){
				enc_len=qp_encode(encoded, buff, bufflen);
			} else {
				if(tenc_struct[j].num == 3){
					enc_len=base64_encode(encoded, buff, bufflen);
				}
			}				
		}
	}
		
  itoa(p, enc_len);
  text[4].campo = p;
	if(flag == 0)		text[10].campo = encoded;
  
  //spedisco l'header
  i=j=0;
  for(k=0; k<=9; k++){
    while((sendtext[i++]=text[k].campo[j++]) != '\0');
    j=0;
    i--;
  }
  
  if(send(sockd, sendtext, i, 0) == -1)							{hcherror("ERROR post_hclib() - write on socket failure : "); return -4;}
  //  spedisco il corpo
	if(send(sockd, text[10].campo, enc_len, 0) == -1)	{hcherror("ERROR post_hchlib() - write on socket failure : "); return -4;}
  
	if(recv(sockd, recvtext, MAX_HEAD, 0) == -1)			{hcherror("ERROR post_hchlib() - read from socket failure : "); return -5;}

  if(sscanf(recvtext, "%s %d %s", ishttp, &code, status) == EOF)       {hcherror("WARNING post_hchlib() - received buffer is empty : "); return -7;}
  if((strcasecmp(ishttp, "HTTP/1.0")!=0) || (code != 200) || (strcasecmp(status, "OK")!=0))	{hcherror("ERROR post_hchlib() - server response is not a 200 OK : "); return -8;}
 
  p1 = strCRLF(recvtext);
  p2 = strCRLF(p1);
  p3 = strCRLF(p2);
  p4 = strCRLF(p3);
  ip1 = p2 - p1;
  ip2 = p3 - p2;
  ip3 = p4 - p3;
  ip4 = strCRLF(p4) - p4;
  p1[ip1-2] = '\0';
  p2[ip2-2] = '\0';
  p3[ip3-2] = '\0';
  p4[ip4-2] = '\0';
  if(sscanf(p1, "%s", cont) == 0) 									hcherror("WARNING post_hchlib() - reading header error : ");
  mime = (char *)&p1[strlen(cont)+1];
  if(sscanf(p2, "%s", cont) == 0)										hcherror("WARNING post_hchlib() - reading header error : ");
  msg_len = atoi((char *)&p2[strlen(cont)+1]);
  if(sscanf(p3, "%s", cont) == 0)										hcherror("WARNING post_hchlib() - reading header error : ");
  strcpy(mimever, (char *)&p3[strlen(cont)+1]);
  if(sscanf(p4, "%s", cont) == 0)										hcherror("WARNING post_hchlib() - reading header error : ");
  strcpy(tenc, (char *)&p4[strlen(cont)+1]);

  if(strcmp(mimever, MIMEver) != 0)									hcherror("WARNING post_hchlib() - bad MIME version : ");
  if((strcasecmp(tenc, text[8].campo) != 0)||(msg_len!=enc_len))	hcherror("WARNING post_hchlib() - server response has an incorrect header : ");
  
  return 0;
}




/*! \brief get_hchlib() - richiede un buffer al server
 *
 * Funzione per richiedere il contenuto di un buffer al server.
 *  Il contenuto del buffer puo' essere binario o testo, a seconda del mimetipe
 *  (dato anche in uscita) verra' decodato
 *
 *  IN:    int sockd        (socket descriptor, creato con create_hchlib()),
 *  OUT:   char *buff       (buffer da RICEVERE),
 *	  		 int *bufflen  		(lunghezza del buffer ricevuto),
 *			 char *tencout	(transfer encoding)
 *	 		   char *mimeout  	(mimetype del dato contenuto in buff)
 *  RETURN: int          (0 se OK, numero negativo se errore, vedi tabella)
 */
int 
get_hchlib(int sockd, char *buff, int *bufflen, char *tencout, char *mimeout)
{
  int recv_byte, msg_len, i, j, ip4,
      flag=0, code, ip1, ip2, ip3, found=0;
  char recv_buff[MAX_NET], mimever[4],
       status[5], ishttp[9],
       mime_p[20], *data, *p4,
       *p1, *p2, *p3, cont[17];

  if(send(sockd, "GET / HTTP/1.0\r\n", strlen("GET / HTTP/1.0\r\n"), 0) == -1)	{hcherror("ERROR - write on socket failure : "); return -4;}

  if((recv_byte=recv(sockd, recv_buff, MAX_NET, MSG_WAITALL)) == 0)  				{hcherror("ERROR - server has closed the connection : "); return -1;}
	
  if(sscanf(recv_buff, "%s %d %s", ishttp, &code, status) == EOF)        	   		{hcherror("WARNING - received buffer is empty : "); return -7;}
  if((strcasecmp(ishttp, "HTTP/1.0")!=0) || (code != 200) || (strcasecmp(status, "OK")!=0))	{hcherror("ERROR - server response is not a 200 OK : "); return -8;}

  p1 = strCRLF(recv_buff);
  p2 = strCRLF(p1);
  p3 = strCRLF(p2);
  p4 = strCRLF(p3);
  ip1 = p2 - p1;
  ip2 = p3 - p2;
  ip3 = p4 - p3;
  ip4 = strCRLF(p4) - p4;
  p1[ip1-2] = '\0';
  p2[ip2-2] = '\0';  
  p3[ip3-2] = '\0';
  p4[ip4-2] = '\0';
  if(sscanf(p1, "%s", cont) == 0)			hcherror("WARNING get_hchlib() - reading header error : ");
  if(strcasecmp(cont, "Content-type:") == 0){
		strcpy(mimeout, (char *)&p1[strlen(cont)+1]);
    found = 1;
  } else {
	  if(strcasecmp(cont, "Content-transfer-encoding:") == 0){
		strcpy(tencout, (char *)&p1[strlen(cont)+1]);
			found = 1;
    } else {
			if(strcasecmp(cont, "Content-lenght:") == 0){
				msg_len = atoi((char *)&p1[strlen(cont)+1]);
				found = 1;
			}else {
			if(strcasecmp(cont, "MIME-Version:") == 0){
					strcpy(mimever, (char *)&p1[strlen(cont)+1]);
				found = 1;
				}
			}
	 	}
  }
  if(sscanf(p2, "%s", cont) == 0)			hcherror("WARNING get_hchlib() - reading header error : ");
  if(strcasecmp(cont, "Content-lenght:") == 0){
    msg_len = atoi((char *)&p2[strlen(cont)+1]);
    found++;
  } else {
		if(strcasecmp(cont, "Content-type:") == 0){
			strcpy(mimeout, (char *)&p2[strlen(cont)+1]);
			found++;
    } else {
		  if(strcasecmp(cont, "Content-transfer-encoding:") == 0){
		strcpy(tencout, (char *)&p2[strlen(cont)+1]);
				found++;
			}
			else {
			if(strcasecmp(cont, "MIME-Version:") == 0){
					strcpy(mimever, (char *)&p2[strlen(cont)+1]);
				found++;
				}
			}
		}
  }
  if(sscanf(p3, "%s", cont) == 0)			hcherror("WARNING get_hchlib() - reading header error : ");
	if(strcasecmp(cont, "Content-transfer-encoding:") == 0){
		strcpy(tencout, (char *)&p3[strlen(cont)+1]);
		found++;
  } else {
    if(strcasecmp(cont, "Content-lenght:") == 0){
      msg_len = atoi((char *)&p3[strlen(cont)+1]);
      found++;
    } else {
			if(strcasecmp(cont, "Content-type:") == 0){
					strcpy(mimeout, (char *)&p3[strlen(cont)+1]);
				found++;
			}else {
			if(strcasecmp(cont, "MIME-Version:") == 0){
					strcpy(mimever, (char *)&p3[strlen(cont)+1]);
				found++;
				}
			}
			
		}
  }
	
	if(sscanf(p4, "%s", cont) == 0)			hcherror("WARNING get_hchlib() - reading header error : ");
	if(strcasecmp(cont, "Content-transfer-encoding:") == 0){
		strcpy(tencout, (char *)&p4[strlen(cont)+1]);
		found++;
  } else {
    if(strcasecmp(cont, "Content-lenght:") == 0){
      msg_len = atoi((char *)&p4[strlen(cont)+1]);
      found++;
    } else {
			if(strcasecmp(cont, "Content-type:") == 0){
					strcpy(mimeout, (char *)&p4[strlen(cont)+1]);
				found++;
			}else {
			if(strcasecmp(cont, "MIME-Version:") == 0){
					strcpy(mimever, (char *)&p4[strlen(cont)+1]);
				found++;
				}
			}
		}
	}
  if(found != 4)				{hcherror("ERROR get_hchlib() - server response has an incorrect header : "); return -9;}
  if(strcmp(mimever, MIMEver) != 0)		{hcherror("WARNING get_hchlib() - bad MIME version :"); return -1;}
  p4[ip4-2] = '\r';  //era stato sovrascritto con '\0'
	
  data = strstr(&p4[ip4-2], "\r\n\r\n");
  data += 4;
	
  strslash(mime_p, mimeout); 
  for(i=0; i<=5; i++){  
    if(strcasecmp(mtype[i].tp, mime_p) == 0){
      flag=1; 
      break;
    }
  }   
  if(flag == 0) 				{hcherror("ERROR get_hchlib() - mimetype unknown : "); return -3; }
	flag = 0;
	for(j=0; j<=4; j++){  
    if(strcasecmp(tenc_struct[j].tp, tencout) == 0){
      flag=1; 
      break;
    }
  } 
	if(flag == 0) 				{hcherror("ERROR get_hchlib() - transfer encoding unknown : "); return -3; }

	if((mtype[i].num == 1) && ((tenc_struct[j].num == 1)||(tenc_struct[j].num == 2)))	hcherror("WARNING get_hchlib() - data may be incorrect : ");

	if(tenc_struct[j].num == 0){
		for(i=0; i<=msg_len; i++)		buff[i]=data[i];
		*bufflen = msg_len;
	}	else {
		if(tenc_struct[j].num == 1){
			for(i=0; i<=msg_len; i++)		buff[i]=data[i];
			*bufflen = msg_len;
		} else {
			if(tenc_struct[j].num == 2){
				*bufflen = qp_decode(buff, data, msg_len);
			} else {
				if(tenc_struct[j].num == 3){
					*bufflen = base64_decode(buff, data, msg_len);
				}
			}				
		}
	}
	buff[*bufflen]='\0';

  return 0;
}


//////////////////////////////
//     funzioni SERVER      //
//////////////////////////////
// backlog stevens lo mette a 1024, per prestazioni ottimali 15-20
int 
listen_hchlib(u_int16_t port, int backlog)
{
  int sd, size, yes=1;
  struct sockaddr_in servaddr;

  size=sizeof(servaddr);
  if((backlog < 0)||(backlog > 1024))             {hcherror("backlog"); return -1;}
  if((sd=socket(AF_INET, SOCK_STREAM, 0)) < 0)    {hcherror("socket()"); return -2;}
  bzero(&servaddr, size);
  servaddr.sin_family=AF_INET;
  servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
  servaddr.sin_port=htons(port);
  if(setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, &yes, sizeof(yes)) == -1)	{hcherror("setsockopt()"); return -3;}
  if(bind(sd, (struct sockaddr *)&servaddr, sizeof(servaddr)) < 0)	{hcherror("bind()"); return -3;}
  
  if(listen(sd, backlog) != 0)										{hcherror("listen()"); return -4;}
  return sd;
}

int 
accept_hchlib(int lsd)
{
  int csd;
	
  if((csd=accept(lsd, (struct sockaddr *) NULL, NULL)) == -1)	{hcerror("accept()"); return -1;}
  return csd; 
}





/*! \brief recv_hchlib() - accetta i POST del client
 *
 * Funzione per ricevere i dati spediti con la post_hchlib() dal client.
 *  Il contenuto del buffer puo' essere binario o testo, a seconda del mimetipe
 *  (dato anche in uscita) verra' decodato
 *
 *  IN:     int sockd        (socket descriptor, creato con listen_hchlib() e connesso da accept_hchlib()),
 *  OUT:	   char *buff       (buffer DA RICEVERE),
 *	   int *bufflen     (lunghezza del buffer ricevuto),
 *	   char *mimeout    (mimetype del dato contenuto in buff)
 *  RETURN: int              (0 se OK, -1 se ha ricevuto GET anziche' POST, numero negativo se errore)
 *
 * nota: chiude il socket
 */
int 
recv_hchlib(int sockd, char *buff, int *bufflen, char *tencout, char *mimeout)
{

  int recv_byte, msg_len, i, j, ip4,
      flag=0, ip1, ip2, ip3, found=0;
  int hv1, hv2;
  char test[5], *data, mime_p[20], tmp[7], *p4, mimever[4],
       send_buff[MAX_HEAD], recv_buff[MAX_ENC],
       *p1, *p2, *p3, cont[17], header[MAX_HEAD];

  // ricevo l'header
  if(recv(sockd, header, MAX_HEAD, 0) == -1)			{hcherror("ERROR recv_hchlib() - remote side has closed the connection : "); return -11;}
  
  if(sscanf(header, "%s / %5s%d.%d", test, tmp, &hv1, &hv2) == EOF) {hcherror("WARNING recv_hchlib() - received buffer is empty : "); return -7;}
  if(strcasecmp(test, "POST") != 0)								return -33;

  if((hv1 > 1) || ((hv1 == 1)&&(hv2 > 0)))				{hcherror("WARNING recv_hchlib() - bad request (incorrect HTTP version) : "); return -1;}

  p1 = strCRLF(header);
  p2 = strCRLF(p1);
  p3 = strCRLF(p2);
  p4 = strCRLF(p3);
  ip1 = p2 - p1;
  ip2 = p3 - p2;
  ip3 = p4 - p3;
  ip4 = strCRLF(p4) - p4;
  p1[ip1-2] = '\0';
  p2[ip2-2] = '\0';  
  p3[ip3-2] = '\0';
  p4[ip4-2] = '\0';
  if(sscanf(p1, "%s", cont) == 0)			hcherror("WARNING recv_hchlib() - reading header error : ");
  if(strcasecmp(cont, "Content-type:") == 0){
		strcpy(mimeout, (char *)&p1[strlen(cont)+1]);
    found = 1;
  } else {
	  if(strcasecmp(cont, "Content-transfer-encoding:") == 0){
		strcpy(tencout, (char *)&p1[strlen(cont)+1]);
			found = 1;
    } else {
		if(strcasecmp(cont, "Content-lenght:") == 0){
				msg_len = atoi((char *)&p1[strlen(cont)+1]);
				found = 1;
			}else {
			if(strcasecmp(cont, "MIME-Version:") == 0){
					strcpy(mimever, (char *)&p1[strlen(cont)+1]);
				found = 1;
				}
			}
			
		}
  }
  if(sscanf(p2, "%s", cont) == 0)			hcherror("WARNING recv_hchlib() - reading header error : ");
  if(strcasecmp(cont, "Content-lenght:") == 0){
    msg_len = atoi((char *)&p2[strlen(cont)+1]);
    found++;
  } else {
		if(strcasecmp(cont, "Content-type:") == 0){
			strcpy(mimeout, (char *)&p2[strlen(cont)+1]);
			found++;
    } else {
		  if(strcasecmp(cont, "Content-transfer-encoding:") == 0){
				strcpy(tencout, (char *)&p2[strlen(cont)+1]);
				found++;
			}
			else {
			if(strcasecmp(cont, "MIME-Version:") == 0){
					strcpy(mimever, (char *)&p2[strlen(cont)+1]);
				found++;
				}
			}
		}
  }
  if(sscanf(p3, "%s", cont) == 0)			hcherror("WARNING recv_hchlib() - reading header error : ");
	if(strcasecmp(cont, "Content-transfer-encoding:") == 0){
		strcpy(tencout, (char *)&p3[strlen(cont)+1]);
		found++;
  } else {
    if(strcasecmp(cont, "Content-lenght:") == 0){
      msg_len = atoi((char *)&p3[strlen(cont)+1]);
      found++;
    } else {
			if(strcasecmp(cont, "Content-type:") == 0){
					strcpy(mimeout, (char *)&p3[strlen(cont)+1]);
				found++;
			}else {
			if(strcasecmp(cont, "MIME-Version:") == 0){
					strcpy(mimever, (char *)&p3[strlen(cont)+1]);
				found++;
				}
			}	
		}
  }
	
	if(sscanf(p4, "%s", cont) == 0)			hcherror("WARNING recv_hchlib() - reading header error : ");
	if(strcasecmp(cont, "Content-transfer-encoding:") == 0){
		strcpy(tencout, (char *)&p4[strlen(cont)+1]);
		found++;
  } else {
    if(strcasecmp(cont, "Content-lenght:") == 0){
      msg_len = atoi((char *)&p4[strlen(cont)+1]);
      found++;
    } else {
			if(strcasecmp(cont, "Content-type:") == 0){
					strcpy(mimeout, (char *)&p4[strlen(cont)+1]);
				found++;
			}else {
			if(strcasecmp(cont, "MIME-Version:") == 0){
					strcpy(mimever, (char *)&p4[strlen(cont)+1]);
				found++;
				}
			}
		}
	}
  if(found != 4)			    		{hcherror("ERROR recv_hchlib() - server response has an incorrect header : "); return -9;}
  if(strcmp(mimever, MIMEver) != 0)		{hcherror("WARNING recv_hchlib() - bad MIME version :"); return -1;}

  // ricevo il corpo
  if(recv(sockd, recv_buff, msg_len, MSG_WAITALL) == -1)			{hcherror("ERROR recv_hchlib() - remote side has closed the connection : "); return -11;}			
  
  sprintf(send_buff, "HTTP/%d.%d 200 OK\r\nContent-type: %s\r\nContent-lenght: %d\r\nMIME-Version: %s\r\nContent-transfer-encoding: %s\r\nServer: %s\r\n\r\n", hv1, hv2, mimeout, msg_len, mimever, tencout, HCHVER);
  if(send(sockd, send_buff, strlen(send_buff), 0) == -1)			{hcherror("ERROR recv_hchlib() - server cannot send a 200 OK : "); return -10;};
  
  if(close(sockd) != 0)								{hcherror("ERROR recv_hchlib() - server can't close the connection : "); return -12;}

  strslash(mime_p, mimeout); 
  for(i=0; i<=5; i++){  
    if(strcasecmp(mtype[i].tp, mime_p) == 0){
      flag=1; 
      break;
    }
  }   
  if(flag == 0) 								{hcherror("ERROR recv_hchlib() - buffer mimetype unknown : "); return -3; }
	flag = 0;
	for(j=0; j<=4; j++){  
    if(strcasecmp(tenc_struct[j].tp, tencout) == 0){
      flag=1; 
      break;
    }
  } 
	if(flag == 0) 								{hcherror("ERROR recv_hchlib() - transfer encoding unknown : "); return -3; }

	if(tenc_struct[j].num == 0){
    for(i=0; i<=msg_len; i++)		buff[i] = recv_buff[i];
		*bufflen = msg_len;
	}	else {
		if(tenc_struct[j].num == 1){
			for(i=0; i<=msg_len; i++)		buff[i]=recv_buff[i];
			*bufflen = msg_len;
		} else {
			if(tenc_struct[j].num == 2){
				*bufflen = qp_decode(buff, recv_buff, msg_len);
			} else {
				if(tenc_struct[j].num == 3){
					*bufflen = base64_decode(buff, recv_buff, msg_len);
				}
			}
		}
	}
	buff[*bufflen]='\0';

  return 0; 
}





/*! \brief send_hchlib() - accetta i GET del client
 *
 * Funzione per spedire i dati richiesti con la get_hchlib() dal client.
 *  Il contenuto del buffer puo' essere binario o testo, a seconda del mimetipe
 *  dato in ingresso verra' encodato
 *
 *  IN:     int sockd        (socket descriptor, creato con listen_hchlib() e connesso da accept_hchlib()),
 *          char *buff       (buffer DA INVIARE),
 *	   int bufflen     (lunghezza del buffer ricevuto),
 *	   char *mime       (mimetype del dato contenuto in buff)
 *  RETURN: int              (0 se OK, -1 se ha ricevuto POST anziche' GET, numero negativo se errore)
 *
 * nota: chiude il socket
 */
int 
send_hchlib(int sockd, void *buff, int bufflen, char *tenc, char *mime)
{
  char encoded[MAX_ENC], 
       recv_buff[MAX_HEAD],
       sendtext[MAX_NET],
       p[8], mime_p[20], test[5], tmp[7]; 
  int flag=0, k,
      h, i, j,
      enc_len, recv_byte,
      recvlen, hv1, hv2;
   
  struct textstr{
    char *campo;
  } text[] = { 
    "HTTP/1.0 200 OK",
    "\r\nContent-type: ",
    "",                      //2  
    "\r\nContent-lenght: ",
    "",                      //4
    "\r\nMIME-Version: ",
    MIMEver,               	 //6
    "\r\nContent-transfer-encoding: ",
    "",                      //8
    "\r\nServer: ",
    HCHVER,
    "\r\n\r\n",
    "",                      //12
  }; 

  if(bufflen > MAX_DATA)					{hcherror("ERROR send_hchlib() - buffer to receive is too long (max is 170kB) : "); return -1;}
  if(strlen(mime) > MAX_MIME)					{hcherror("ERROR send_hchlib() - mimetype is too long (max is 64chars) : "); return -2;}
  if(strlen(tenc) > MAX_TENC)					{hcherror("ERROR send_hchlib() - bad transfer encoding : "); return -2;}

  strslash(mime_p, mime); 
  for(i=0; i<=5; i++){  
    if(strcasecmp(mtype[i].tp, mime_p) == 0){
      flag=1; 
      break;
    }
  }   
  if(flag == 0) 						{hcherror("ERROR send_hchlib() - mimetype unknown : "); return -3; }
  text[2].campo = mime;
  flag=0;
  for(j=0; j<=4; j++){  
    if(strcasecmp(tenc_struct[j].tp, tenc) == 0){
      flag=1; 
      break;
    }
  } 
  if(flag == 0) 									{hcherror("ERROR send_hchlib() - transfer encoding unknown : "); return -3; }
  text[8].campo = tenc;
 
  if((mtype[i].num == 1) && ((tenc_struct[j].num == 1)||(tenc_struct[j].num == 2)))	hcherror("WARNING send_hchlib() - data may be lost with this encoding : ");	

  flag = 0;
  if(tenc_struct[j].num == 0){
		enc_len = bufflen;
		text[12].campo = buff;
		flag = 1;
	}	else {
		if(tenc_struct[j].num == 1){
			enc_len=break_line(encoded, buff, bufflen);
		} else {
			if(tenc_struct[j].num == 2){
				enc_len=qp_encode(encoded, buff, bufflen);
			} else {
				if(tenc_struct[j].num == 3){
					enc_len=base64_encode(encoded, buff, bufflen);
				}
			}				
		}
	}
	itoa(p, enc_len);
  text[4].campo = p;
	
	if(flag == 0)		text[12].campo = encoded;
 	
  i=j=0;
  for(k=0; k<=12; k++){
    while((sendtext[i++]=text[k].campo[j++]) != '\0');
    j=0;
    i--;
  }
	
  if((recv_byte=recv(sockd, recv_buff, MAX_HEAD, 0)) == -1)		{hcherror("ERROR send_hchlib()- read from socket failure : "); return -2;}
  recv_buff[recv_byte] = '\0';
   
  if(sscanf(recv_buff, "%s / %5s%d.%d", test, tmp, &hv1, &hv2) <= 0) 	{hcherror("WARNING send_hchlib() - bad request : "); return -1;}
  if(strcasecmp(test, "GET") != 0)								return -33;
  if((hv1 > 1) || ((hv1 == 1)&&(hv2 > 0)))				{hcherror("WARNING send_hchlib() - bad request : "); return -1;}

  if(send(sockd, sendtext, i, 0) == -1)					{hcherror("ERROR send_hchlib() - write on socket failure : "); return -4;}
  if(close(sockd) != 0)														{hcerror("ERROR send_hclib() - server can't close the connection : "); return -12;}
  
  return 0;	
}


/* 
 * EOF 
 */
