/*! \file hchlib_encode.h
 * httpchanlib 1.0 encoding/decoding function file
 * This source code is distributed under GPL GNU Public License version 2 or later.
 */

#ifndef _HCHLIB_ENCODE_H
#define _HCHLIB_ENCODE_H

/*! \brief break line
 *
 * Funzione per l'interruzione delle linee di un testo a meno di 76 caratteri
 *
 * IN: 	char *dst (puntatore stringa di destinazione),
 *    	  const char *src (puntatore al dato di origine),
 *    	  unsigned int len (lunghezza dato di origine)
 *
 * RETURN: int (lunghezza stringa di destinazione)
 */
int
break_line(char *dst, const char *src, unsigned int len)
{
  int x, y, z, m;
  char c;
  x=z=y=m=0;
  while(x < len){
		c = dst[z] = src[x];

		if(c == ' ')  m = y;
		if((c == '\r')||(c == '\n'))	y=0;	
		if(y == (MAXLINE-1)){
			if(m == 0){
				dst[z] = '\r';
				dst[z+1] = '\n';
				z++;
			} else {
				while(src[x] != ' '){
					x--;
					z--;
				}
				dst[z] = '\r';
				dst[z+1] = '\n';
				z++;
			}
			y=m=0;
		}		
		x++;
		z++;
		y++;
  }
  dst[z--] = '\0';
  return z;
}


/*! \brief Encoding base64
 *
 * Funzione per l'encoding di una stringa in base64
 *
 * IN: 	  char *dst (puntatore stringa di destinazione),
 *    	  const char *src (puntatore al dato di origine),
 *    	  unsigned int len (lunghezza dato di origine)
 *
 * RETURN: int (lunghezza stringa di destinazione)
 */
int 
base64_encode(char *dst, const char *src, unsigned int len)
{
  int x, y = 0, i, flag=0, m=0;
  int n = 3;
  char triple[3], quad[4], c;
 
  for(x = 0; x < len; x = x + 3){
    if((len - x) / 3 == 0)      n = (len - x) % 3;
 
    for(i=0; i < 3; i++)  triple[i] = '0';
    for(i=0; i < n; i++)  triple[i] = src[x + i];
    quad[0] = base64_table[(triple[0] & 0xFC) >> 2];   // FC = 11111100
    quad[1] = base64_table[((triple[0] & 0x03) << 4) | ((triple[1] & 0xF0) >> 4)];  // 03 = 11
    quad[2] = base64_table[((triple[1] & 0x0F) << 2) | ((triple[2] & 0xC0) >> 6)];  // 0F = 1111, C0=11110
    quad[3] = base64_table[triple[2] & 0x3F];          // 3F = 111111
    if(n < 3)  quad[3] = '=';
    if(n < 2)  quad[2] = '=';
    for(i=0; i < 4; i++)  dst[y + i] = quad[i];
    y = y + 4;
    m = m + 4;
    if((m != 0)&&((m % MAXLINE) == 0)){
      dst[y] = '\r';
      dst[y+1] = '\n';
      flag++;
      y += 2;
      m = 0;
    }
  }
  dst[y] = '\0';
  return y;
}

/*! \brief Decoding base64 - scelta carattere
 */
char 
decode(char c)
{
  if(c >= 'A' && c <= 'Z')     c = c - 'A';
  else 
   if(c >= 'a' && c <= 'z')     c = c - 'a' + 26;
   else
    if(c >= '0' && c <= '9')     c = c - '0' + 52;
    else 
     if(c == '+')      c = 62;
     else 
      if(c == '/')      c = 63; 
      else     c = 0;
  return c;
}





/*! \brief Decoding base64
*
* Funzione per il decoding di un dato in base64
*
* IN: 	  char *dst (puntatore dato di destinazione),
*    	  const char *src (puntatore alla stringa origine),
*    	  unsigned int len (lunghezza stringa di origine)
*
* RETURN: int (lunghezza dato di destinazione)
*/
int
base64_decode(char *dst, const char *src, unsigned int len)
{
  int x, y = 0, n, i, flag=0, m=0;
  char triple[3];
  char quad[4];
 
  for(x = 0; x < (len - m); x = x + 4){
      memset(quad, 0, 4);
      n = 4;
      for(i=0; i < n; i++){
        if(src[x + i + m] == '\r')  m += 2;
        if(src[x + i + m] == '=')  flag++;
        quad[i] = src[x + i + m];
      }

      quad[0]=decode(quad[0]);
      quad[1]=decode(quad[1]);
      quad[2]=decode(quad[2]);
      quad[3]=decode(quad[3]);
      triple[0] = (quad[0] << 2) | (quad[1] >> 4);
      triple[1] = ((quad[1] << 4) & 0xF0) | (quad[2] >> 2);   // F0 = 1111 0000
      triple[2] = ((quad[2] << 6) & 0xC0) | quad[3];          // C0 = 1100 0000

      dst[y] = triple[0];
      dst[y+1] = triple[1];
      dst[y+2] = triple[2];
      y += 3;
    }
    for(i=0; i <= flag; i++)  dst[y-i] = '\0';
   return y - flag;
}





/*! \brief Encoding quoted printable
 *
 * Funzione per il encoding di una stringa in quoted printable
 *
 * IN: 	  char *dst (puntatore stringa di destinazione),
 *    	  const char *src (puntatore alla stringa origine),
 *    	  unsigned int len (lunghezza stringa di origine)
 *
 * RETURN: int (lunghezza stringa di destinazione)
 */
int
qp_encode(char *dst, const char *src, unsigned int len)
{
  int i = 0, j = 0, m;
  char tmp[4];
	
  while (i <= len) {
    if (src[i] < 0) {
      for (m = 0; m <= qp[src[i] + 256].lench; m++) {
	dst[j + m] = qp[src[i] + 256].ch[m];
      }
      j = j + qp[src[i] + 256].lench;
    } else {
      for (m = 0; m <= qp[src[i]].lench; m++) {
	dst[j + m] = qp[src[i]].ch[m];
      }
      j = j + qp[src[i]].lench;
    }
    i++;
    if ((j%MAXLINE) >=73) {
      dst[j] = '=';
      dst[j + 1] = '\r';
      dst[j + 2] = '\n';
      j = j + 3;
    }
  }
  return j;
}





/*! \brief Decoding quoted printable
 *
 * Funzione per il decoding di una stringa in quoted printable
 *
 * IN: 	  char *dst (puntatore stringa di destinazione),
 *    	  const char *src (puntatore alla stringa origine),
 *    	  unsigned int len (lunghezza stringa di origine)
 *
 * RETURN: lunghezza stringa di destinazione
 */
int 
qp_decode(char *dst, const char *src, unsigned int len)
{ 
  int i = 0, j = 0, m, flag = 0; 
  char tmp[4]; 
	
  while (i <= len) {
    if(src[i] == '='){
      if((src[i+1] == '\r')||(src[i+1] == '\n')){  //'\n' a causa di MAX_LINE
        if(src[i+2] == '\n'){
          dst[j] = ' ';
          i = i + 3; 
          j--;  
        } else {
          dst[j] = ' ';
          i = i + 2;	
          j--;  
        }
      } else {           		//deve essere un carattere
        tmp[0] = '=';
        tmp[1] = src[i+1];
        tmp[2] = src[i+2];
        tmp[3] = '\0';
        for(m=0; m <= 256; m++){
          if(strncmp(tmp, qp[m].ch, 3) == 0){
            if(m <= 127){
              dst[j] = m;
            } else {
              dst[j] = m - 256; 
            } 
           flag = 1;
           break;   			//esce dal for(), evita di processare dati inutili
          }
        } 
        if(flag == 0){
         return -1;  			//carattere non trovato
        }
        i = i + 3; 
      } 
    } else {
      dst[j] = src[i];
      i++;
    }
    j++;
  }
  return j-2;
} 

#endif

/*
 * EOF
 */
